package java.time.temporal;

import java.util.ResourceBundle;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.time.DateTimeException;
import java.time.format.ResolverStyle;
import java.util.Map;
import java.time.chrono.ChronoLocalDate;
import java.time.chrono.Chronology;
import java.util.concurrent.ConcurrentHashMap;
import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import sun.util.locale.provider.CalendarDataUtility;
import java.util.Objects;
import java.util.Locale;
import java.time.DayOfWeek;
import java.util.concurrent.ConcurrentMap;
import java.io.Serializable;

public final class WeekFields implements Serializable
{
    private static final ConcurrentMap<String, WeekFields> CACHE;
    public static final WeekFields ISO;
    public static final WeekFields SUNDAY_START;
    public static final TemporalUnit WEEK_BASED_YEARS;
    private static final long serialVersionUID = -1177360819670808121L;
    private final DayOfWeek firstDayOfWeek;
    private final int minimalDays;
    private final transient TemporalField dayOfWeek;
    private final transient TemporalField weekOfMonth;
    private final transient TemporalField weekOfYear;
    private final transient TemporalField weekOfWeekBasedYear;
    private final transient TemporalField weekBasedYear;
    
    public static WeekFields of(Locale locale) {
        Objects.requireNonNull(locale, "locale");
        locale = new Locale(locale.getLanguage(), locale.getCountry());
        return of(DayOfWeek.SUNDAY.plus((long)(CalendarDataUtility.retrieveFirstDayOfWeek(locale) - 1)), CalendarDataUtility.retrieveMinimalDaysInFirstWeek(locale));
    }
    
    public static WeekFields of(DayOfWeek dayOfWeek, int i) {
        String string = dayOfWeek.toString() + i;
        WeekFields weekFields = (WeekFields)WeekFields.CACHE.get(string);
        if (weekFields == null) {
            WeekFields.CACHE.putIfAbsent(string, new WeekFields(dayOfWeek, i));
            weekFields = (WeekFields)WeekFields.CACHE.get(string);
            return weekFields;
        }
        return weekFields;
    }
    
    private WeekFields(DayOfWeek dayOfWeek, int minimalDays) {
        this.dayOfWeek = WeekFields.ComputedDayOfField.ofDayOfWeekField(this);
        this.weekOfMonth = WeekFields.ComputedDayOfField.ofWeekOfMonthField(this);
        this.weekOfYear = WeekFields.ComputedDayOfField.ofWeekOfYearField(this);
        this.weekOfWeekBasedYear = WeekFields.ComputedDayOfField.ofWeekOfWeekBasedYearField(this);
        this.weekBasedYear = WeekFields.ComputedDayOfField.ofWeekBasedYearField(this);
        Objects.requireNonNull(dayOfWeek, "firstDayOfWeek");
        if (minimalDays < 1 || minimalDays > 7) {
            throw new IllegalArgumentException("Minimal number of days is invalid");
        }
        this.firstDayOfWeek = dayOfWeek;
        this.minimalDays = minimalDays;
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException, InvalidObjectException {
        objectInputStream.defaultReadObject();
        if (this.firstDayOfWeek == null) {
            throw new InvalidObjectException("firstDayOfWeek is null");
        }
        if (this.minimalDays < 1 || this.minimalDays > 7) {
            throw new InvalidObjectException("Minimal number of days is invalid");
        }
        return;
    }
    
    private Object readResolve() throws InvalidObjectException {
        try {
            return of(this.firstDayOfWeek, this.minimalDays);
        }
        catch (IllegalArgumentException ex) {
            throw new InvalidObjectException("Invalid serialized WeekFields: " + ex.getMessage());
        }
    }
    
    public DayOfWeek getFirstDayOfWeek() {
        return this.firstDayOfWeek;
    }
    
    public int getMinimalDaysInFirstWeek() {
        return this.minimalDays;
    }
    
    public TemporalField dayOfWeek() {
        return this.dayOfWeek;
    }
    
    public TemporalField weekOfMonth() {
        return this.weekOfMonth;
    }
    
    public TemporalField weekOfYear() {
        return this.weekOfYear;
    }
    
    public TemporalField weekOfWeekBasedYear() {
        return this.weekOfWeekBasedYear;
    }
    
    public TemporalField weekBasedYear() {
        return this.weekBasedYear;
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof WeekFields && this.hashCode() == o.hashCode());
    }
    
    @Override
    public int hashCode() {
        return this.firstDayOfWeek.ordinal() * 7 + this.minimalDays;
    }
    
    @Override
    public String toString() {
        return "WeekFields[" + this.firstDayOfWeek + 44 + this.minimalDays + 93;
    }
    
    static {
        CACHE = new ConcurrentHashMap(4, 0.75f, 2);
        ISO = new WeekFields(DayOfWeek.MONDAY, 4);
        SUNDAY_START = of(DayOfWeek.SUNDAY, 1);
        WEEK_BASED_YEARS = IsoFields.WEEK_BASED_YEARS;
    }
    
    static class ComputedDayOfField implements TemporalField
    {
        private final String name;
        private final WeekFields weekDef;
        private final TemporalUnit baseUnit;
        private final TemporalUnit rangeUnit;
        private final ValueRange range;
        private static final ValueRange DAY_OF_WEEK_RANGE;
        private static final ValueRange WEEK_OF_MONTH_RANGE;
        private static final ValueRange WEEK_OF_YEAR_RANGE;
        private static final ValueRange WEEK_OF_WEEK_BASED_YEAR_RANGE;
        
        static WeekFields.ComputedDayOfField ofDayOfWeekField(WeekFields weekFields) {
            return new WeekFields.ComputedDayOfField("DayOfWeek", weekFields, ChronoUnit.DAYS, ChronoUnit.WEEKS, WeekFields.ComputedDayOfField.DAY_OF_WEEK_RANGE);
        }
        
        static WeekFields.ComputedDayOfField ofWeekOfMonthField(WeekFields weekFields) {
            return new WeekFields.ComputedDayOfField("WeekOfMonth", weekFields, ChronoUnit.WEEKS, ChronoUnit.MONTHS, WeekFields.ComputedDayOfField.WEEK_OF_MONTH_RANGE);
        }
        
        static WeekFields.ComputedDayOfField ofWeekOfYearField(WeekFields weekFields) {
            return new WeekFields.ComputedDayOfField("WeekOfYear", weekFields, ChronoUnit.WEEKS, ChronoUnit.YEARS, WeekFields.ComputedDayOfField.WEEK_OF_YEAR_RANGE);
        }
        
        static WeekFields.ComputedDayOfField ofWeekOfWeekBasedYearField(WeekFields weekFields) {
            return new WeekFields.ComputedDayOfField("WeekOfWeekBasedYear", weekFields, ChronoUnit.WEEKS, IsoFields.WEEK_BASED_YEARS, WeekFields.ComputedDayOfField.WEEK_OF_WEEK_BASED_YEAR_RANGE);
        }
        
        static WeekFields.ComputedDayOfField ofWeekBasedYearField(WeekFields weekFields) {
            return new WeekFields.ComputedDayOfField("WeekBasedYear", weekFields, IsoFields.WEEK_BASED_YEARS, ChronoUnit.FOREVER, ChronoField.YEAR.range());
        }
        
        private ChronoLocalDate ofWeekBasedYear(Chronology chronology, int n, int min, int n2) {
            ChronoLocalDate date = chronology.date(n, 1, 1);
            int startOfWeekOffset = this.startOfWeekOffset(1, this.localizedDayOfWeek(date));
            min = Math.min(min, this.computeWeek(startOfWeekOffset, date.lengthOfYear() + this.weekDef.getMinimalDaysInFirstWeek()) - 1);
            return date.plus((long)(-startOfWeekOffset + (n2 - 1) + (min - 1) * 7), ChronoUnit.DAYS);
        }
        
        private ComputedDayOfField(String name, WeekFields weekDef, TemporalUnit baseUnit, TemporalUnit rangeUnit, ValueRange range) {
            this.name = name;
            this.weekDef = weekDef;
            this.baseUnit = baseUnit;
            this.rangeUnit = rangeUnit;
            this.range = range;
        }
        
        @Override
        public long getFrom(TemporalAccessor temporalAccessor) {
            if (this.rangeUnit == ChronoUnit.WEEKS) {
                return (long)this.localizedDayOfWeek(temporalAccessor);
            }
            if (this.rangeUnit == ChronoUnit.MONTHS) {
                return this.localizedWeekOfMonth(temporalAccessor);
            }
            if (this.rangeUnit == ChronoUnit.YEARS) {
                return this.localizedWeekOfYear(temporalAccessor);
            }
            if (this.rangeUnit == WeekFields.WEEK_BASED_YEARS) {
                return (long)this.localizedWeekOfWeekBasedYear(temporalAccessor);
            }
            if (this.rangeUnit == ChronoUnit.FOREVER) {
                return (long)this.localizedWeekBasedYear(temporalAccessor);
            }
            throw new IllegalStateException("unreachable, rangeUnit: " + this.rangeUnit + ", this: " + this);
        }
        
        private int localizedDayOfWeek(TemporalAccessor temporalAccessor) {
            return Math.floorMod(temporalAccessor.get(ChronoField.DAY_OF_WEEK) - this.weekDef.getFirstDayOfWeek().getValue(), 7) + 1;
        }
        
        private int localizedDayOfWeek(int n) {
            return Math.floorMod(n - this.weekDef.getFirstDayOfWeek().getValue(), 7) + 1;
        }
        
        private long localizedWeekOfMonth(TemporalAccessor temporalAccessor) {
            int localizedDayOfWeek = this.localizedDayOfWeek(temporalAccessor);
            int value = temporalAccessor.get(ChronoField.DAY_OF_MONTH);
            return (long)this.computeWeek(this.startOfWeekOffset(value, localizedDayOfWeek), value);
        }
        
        private long localizedWeekOfYear(TemporalAccessor temporalAccessor) {
            int localizedDayOfWeek = this.localizedDayOfWeek(temporalAccessor);
            int value = temporalAccessor.get(ChronoField.DAY_OF_YEAR);
            return (long)this.computeWeek(this.startOfWeekOffset(value, localizedDayOfWeek), value);
        }
        
        private int localizedWeekBasedYear(TemporalAccessor temporalAccessor) {
            int localizedDayOfWeek = this.localizedDayOfWeek(temporalAccessor);
            int value = temporalAccessor.get(ChronoField.YEAR);
            int value2 = temporalAccessor.get(ChronoField.DAY_OF_YEAR);
            int startOfWeekOffset = this.startOfWeekOffset(value2, localizedDayOfWeek);
            int computeWeek = this.computeWeek(startOfWeekOffset, value2);
            if (computeWeek == 0) {
                return value - 1;
            }
            if (computeWeek >= this.computeWeek(startOfWeekOffset, (int)temporalAccessor.range(ChronoField.DAY_OF_YEAR).getMaximum() + this.weekDef.getMinimalDaysInFirstWeek())) {
                return value + 1;
            }
            return value;
        }
        
        private int localizedWeekOfWeekBasedYear(TemporalAccessor temporal) {
            int localizedDayOfWeek = this.localizedDayOfWeek(temporal);
            int value = temporal.get(ChronoField.DAY_OF_YEAR);
            int startOfWeekOffset = this.startOfWeekOffset(value, localizedDayOfWeek);
            int computeWeek = this.computeWeek(startOfWeekOffset, value);
            if (computeWeek == 0) {
                return this.localizedWeekOfWeekBasedYear(Chronology.from(temporal).date(temporal).minus((long)value, ChronoUnit.DAYS));
            }
            if (computeWeek > 50) {
                int computeWeek2 = this.computeWeek(startOfWeekOffset, (int)temporal.range(ChronoField.DAY_OF_YEAR).getMaximum() + this.weekDef.getMinimalDaysInFirstWeek());
                if (computeWeek >= computeWeek2) {
                    computeWeek = computeWeek - computeWeek2 + 1;
                }
            }
            return computeWeek;
        }
        
        private int startOfWeekOffset(int n, int n2) {
            int floorMod = Math.floorMod(n - n2, 7);
            int n3 = -floorMod;
            if (floorMod + 1 > this.weekDef.getMinimalDaysInFirstWeek()) {
                n3 = 7 - floorMod;
            }
            return n3;
        }
        
        private int computeWeek(int n, int n2) {
            return (7 + n + (n2 - 1)) / 7;
        }
        
        @Override
        public <R extends Temporal> R adjustInto(R temporal, long n) {
            int checkValidIntValue = this.range.checkValidIntValue(n, this);
            int value = temporal.get(this);
            if (checkValidIntValue == value) {
                return temporal;
            }
            if (this.rangeUnit == ChronoUnit.FOREVER) {
                return this.ofWeekBasedYear(Chronology.from(temporal), (int)n, temporal.get(this.weekDef.weekOfWeekBasedYear), temporal.get(this.weekDef.dayOfWeek));
            }
            return temporal.plus((long)(checkValidIntValue - value), this.baseUnit);
        }
        
        @Override
        public ChronoLocalDate resolve(Map<TemporalField, Long> map, TemporalAccessor temporal, ResolverStyle resolverStyle) {
            long longValue = (long)map.get(this);
            int intExact = Math.toIntExact(longValue);
            if (this.rangeUnit == ChronoUnit.WEEKS) {
                long l = (long)(Math.floorMod(this.weekDef.getFirstDayOfWeek().getValue() - 1 + (this.range.checkValidIntValue(longValue, this) - 1), 7) + 1);
                map.remove(this);
                map.put(ChronoField.DAY_OF_WEEK, Long.valueOf(l));
                return null;
            }
            if (!map.containsKey(ChronoField.DAY_OF_WEEK)) {
                return null;
            }
            int localizedDayOfWeek = this.localizedDayOfWeek(ChronoField.DAY_OF_WEEK.checkValidIntValue((long)map.get(ChronoField.DAY_OF_WEEK)));
            Chronology from = Chronology.from(temporal);
            if (map.containsKey(ChronoField.YEAR)) {
                int checkValidIntValue = ChronoField.YEAR.checkValidIntValue((long)map.get(ChronoField.YEAR));
                if (this.rangeUnit == ChronoUnit.MONTHS && map.containsKey(ChronoField.MONTH_OF_YEAR)) {
                    return this.resolveWoM(map, from, checkValidIntValue, (long)map.get(ChronoField.MONTH_OF_YEAR), (long)intExact, localizedDayOfWeek, resolverStyle);
                }
                if (this.rangeUnit == ChronoUnit.YEARS) {
                    return this.resolveWoY(map, from, checkValidIntValue, (long)intExact, localizedDayOfWeek, resolverStyle);
                }
            }
            else if ((this.rangeUnit == WeekFields.WEEK_BASED_YEARS || this.rangeUnit == ChronoUnit.FOREVER) && map.containsKey(this.weekDef.weekBasedYear) && map.containsKey(this.weekDef.weekOfWeekBasedYear)) {
                return this.resolveWBY(map, from, localizedDayOfWeek, resolverStyle);
            }
            return null;
        }
        
        private ChronoLocalDate resolveWoM(Map<TemporalField, Long> map, Chronology chronology, int n, long x, long x2, int n2, ResolverStyle resolverStyle) {
            ChronoLocalDate chronoLocalDate;
            if (resolverStyle == ResolverStyle.LENIENT) {
                ChronoLocalDate plus = chronology.date(n, 1, 1).plus(Math.subtractExact(x, 1L), ChronoUnit.MONTHS);
                chronoLocalDate = plus.plus(Math.addExact(Math.multiplyExact(Math.subtractExact(x2, this.localizedWeekOfMonth(plus)), 7L), (long)(n2 - this.localizedDayOfWeek(plus))), ChronoUnit.DAYS);
            }
            else {
                ChronoLocalDate date = chronology.date(n, ChronoField.MONTH_OF_YEAR.checkValidIntValue(x), 1);
                chronoLocalDate = date.plus((long)((int)((long)this.range.checkValidIntValue(x2, this) - this.localizedWeekOfMonth(date)) * 7 + (n2 - this.localizedDayOfWeek(date))), ChronoUnit.DAYS);
                if (resolverStyle == ResolverStyle.STRICT && chronoLocalDate.getLong(ChronoField.MONTH_OF_YEAR) != x) {
                    throw new DateTimeException("Strict mode rejected resolved date as it is in a different month");
                }
            }
            map.remove(this);
            map.remove(ChronoField.YEAR);
            map.remove(ChronoField.MONTH_OF_YEAR);
            map.remove(ChronoField.DAY_OF_WEEK);
            return chronoLocalDate;
        }
        
        private ChronoLocalDate resolveWoY(Map<TemporalField, Long> map, Chronology chronology, int n, long x, int n2, ResolverStyle resolverStyle) {
            ChronoLocalDate date = chronology.date(n, 1, 1);
            ChronoLocalDate chronoLocalDate;
            if (resolverStyle == ResolverStyle.LENIENT) {
                chronoLocalDate = date.plus(Math.addExact(Math.multiplyExact(Math.subtractExact(x, this.localizedWeekOfYear(date)), 7L), (long)(n2 - this.localizedDayOfWeek(date))), ChronoUnit.DAYS);
            }
            else {
                chronoLocalDate = date.plus((long)((int)((long)this.range.checkValidIntValue(x, this) - this.localizedWeekOfYear(date)) * 7 + (n2 - this.localizedDayOfWeek(date))), ChronoUnit.DAYS);
                if (resolverStyle == ResolverStyle.STRICT && chronoLocalDate.getLong(ChronoField.YEAR) != (long)n) {
                    throw new DateTimeException("Strict mode rejected resolved date as it is in a different year");
                }
            }
            map.remove(this);
            map.remove(ChronoField.YEAR);
            map.remove(ChronoField.DAY_OF_WEEK);
            return chronoLocalDate;
        }
        
        private ChronoLocalDate resolveWBY(Map<TemporalField, Long> map, Chronology chronology, int n, ResolverStyle resolverStyle) {
            int checkValidIntValue = this.weekDef.weekBasedYear.range().checkValidIntValue((long)map.get(this.weekDef.weekBasedYear), this.weekDef.weekBasedYear);
            ChronoLocalDate chronoLocalDate;
            if (resolverStyle == ResolverStyle.LENIENT) {
                chronoLocalDate = this.ofWeekBasedYear(chronology, checkValidIntValue, 1, n).plus(Math.subtractExact((long)map.get(this.weekDef.weekOfWeekBasedYear), 1L), ChronoUnit.WEEKS);
            }
            else {
                chronoLocalDate = this.ofWeekBasedYear(chronology, checkValidIntValue, this.weekDef.weekOfWeekBasedYear.range().checkValidIntValue((long)map.get(this.weekDef.weekOfWeekBasedYear), this.weekDef.weekOfWeekBasedYear), n);
                if (resolverStyle == ResolverStyle.STRICT && this.localizedWeekBasedYear(chronoLocalDate) != checkValidIntValue) {
                    throw new DateTimeException("Strict mode rejected resolved date as it is in a different week-based-year");
                }
            }
            map.remove(this);
            map.remove(this.weekDef.weekBasedYear);
            map.remove(this.weekDef.weekOfWeekBasedYear);
            map.remove(ChronoField.DAY_OF_WEEK);
            return chronoLocalDate;
        }
        
        @Override
        public String getDisplayName(Locale obj) {
            Objects.requireNonNull(obj, "locale");
            if (this.rangeUnit == ChronoUnit.YEARS) {
                ResourceBundle javaTimeFormatData = LocaleProviderAdapter.getResourceBundleBased().getLocaleResources(obj).getJavaTimeFormatData();
                return javaTimeFormatData.containsKey("field.week") ? javaTimeFormatData.getString("field.week") : this.name;
            }
            return this.name;
        }
        
        @Override
        public TemporalUnit getBaseUnit() {
            return this.baseUnit;
        }
        
        @Override
        public TemporalUnit getRangeUnit() {
            return this.rangeUnit;
        }
        
        @Override
        public boolean isDateBased() {
            return true;
        }
        
        @Override
        public boolean isTimeBased() {
            return false;
        }
        
        @Override
        public ValueRange range() {
            return this.range;
        }
        
        @Override
        public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
            if (temporalAccessor.isSupported(ChronoField.DAY_OF_WEEK)) {
                if (this.rangeUnit == ChronoUnit.WEEKS) {
                    return true;
                }
                if (this.rangeUnit == ChronoUnit.MONTHS) {
                    return temporalAccessor.isSupported(ChronoField.DAY_OF_MONTH);
                }
                if (this.rangeUnit == ChronoUnit.YEARS) {
                    return temporalAccessor.isSupported(ChronoField.DAY_OF_YEAR);
                }
                if (this.rangeUnit == WeekFields.WEEK_BASED_YEARS) {
                    return temporalAccessor.isSupported(ChronoField.DAY_OF_YEAR);
                }
                if (this.rangeUnit == ChronoUnit.FOREVER) {
                    return temporalAccessor.isSupported(ChronoField.YEAR);
                }
            }
            return false;
        }
        
        @Override
        public ValueRange rangeRefinedBy(TemporalAccessor temporalAccessor) {
            if (this.rangeUnit == ChronoUnit.WEEKS) {
                return this.range;
            }
            if (this.rangeUnit == ChronoUnit.MONTHS) {
                return this.rangeByWeek(temporalAccessor, ChronoField.DAY_OF_MONTH);
            }
            if (this.rangeUnit == ChronoUnit.YEARS) {
                return this.rangeByWeek(temporalAccessor, ChronoField.DAY_OF_YEAR);
            }
            if (this.rangeUnit == WeekFields.WEEK_BASED_YEARS) {
                return this.rangeWeekOfWeekBasedYear(temporalAccessor);
            }
            if (this.rangeUnit == ChronoUnit.FOREVER) {
                return ChronoField.YEAR.range();
            }
            throw new IllegalStateException("unreachable, rangeUnit: " + this.rangeUnit + ", this: " + this);
        }
        
        private ValueRange rangeByWeek(TemporalAccessor temporalAccessor, TemporalField temporalField) {
            int startOfWeekOffset = this.startOfWeekOffset(temporalAccessor.get(temporalField), this.localizedDayOfWeek(temporalAccessor));
            ValueRange range = temporalAccessor.range(temporalField);
            return ValueRange.of((long)this.computeWeek(startOfWeekOffset, (int)range.getMinimum()), (long)this.computeWeek(startOfWeekOffset, (int)range.getMaximum()));
        }
        
        private ValueRange rangeWeekOfWeekBasedYear(TemporalAccessor temporalAccessor) {
            if (!temporalAccessor.isSupported(ChronoField.DAY_OF_YEAR)) {
                return WeekFields.ComputedDayOfField.WEEK_OF_YEAR_RANGE;
            }
            int localizedDayOfWeek = this.localizedDayOfWeek(temporalAccessor);
            int value = temporalAccessor.get(ChronoField.DAY_OF_YEAR);
            int startOfWeekOffset = this.startOfWeekOffset(value, localizedDayOfWeek);
            int computeWeek = this.computeWeek(startOfWeekOffset, value);
            if (computeWeek == 0) {
                return this.rangeWeekOfWeekBasedYear(Chronology.from(temporalAccessor).date(temporalAccessor).minus((long)(value + 7), ChronoUnit.DAYS));
            }
            int n = (int)temporalAccessor.range(ChronoField.DAY_OF_YEAR).getMaximum();
            int computeWeek2 = this.computeWeek(startOfWeekOffset, n + this.weekDef.getMinimalDaysInFirstWeek());
            if (computeWeek >= computeWeek2) {
                return this.rangeWeekOfWeekBasedYear(Chronology.from(temporalAccessor).date(temporalAccessor).plus((long)(n - value + 1 + 7), ChronoUnit.DAYS));
            }
            return ValueRange.of(1L, (long)(computeWeek2 - 1));
        }
        
        @Override
        public String toString() {
            return this.name + "[" + this.weekDef.toString() + "]";
        }
        
        static {
            DAY_OF_WEEK_RANGE = ValueRange.of(1L, 7L);
            WEEK_OF_MONTH_RANGE = ValueRange.of(0L, 1L, 4L, 6L);
            WEEK_OF_YEAR_RANGE = ValueRange.of(0L, 1L, 52L, 54L);
            WEEK_OF_WEEK_BASED_YEAR_RANGE = ValueRange.of(1L, 52L, 53L);
        }
    }
}
